@@ -175,7 +175,6 @@ HighsInt Highs_versionMajor(void) { return highsVersionMajor(); }
 HighsInt Highs_versionMinor(void) { return highsVersionMinor(); }
 HighsInt Highs_versionPatch(void) { return highsVersionPatch(); }
 const char* Highs_githash(void) { return highsGithash(); }
-const char* Highs_compilationDate(void) { return highsCompilationDate(); }
 
 HighsInt Highs_presolve(void* highs) {
   return (HighsInt)((Highs*)highs)->presolve();
@@ -224,6 +223,10 @@ HighsInt Highs_writeModel(void* highs, const char* filename) {
   return (HighsInt)((Highs*)highs)->writeModel(std::string(filename));
 }
 
+HighsInt Highs_writePresolvedModel(void* highs, const char* filename) {
+  return (HighsInt)((Highs*)highs)->writePresolvedModel(std::string(filename));
+}
+
 HighsInt Highs_writeSolution(const void* highs, const char* filename) {
   return (HighsInt)((Highs*)highs)
       ->writeSolution(std::string(filename), kSolutionStyleRaw);
@@ -1097,14 +1100,20 @@ HighsInt Highs_getHighsLpData(const HighsLp& lp, const HighsInt a_format,
   *offset = lp.offset_;
   *num_col = lp.num_col_;
   *num_row = lp.num_row_;
+  *num_nz = 0;  // In case one of the matrix dimensions is zero
   if (*num_col > 0) {
-    memcpy(col_cost, lp.col_cost_.data(), *num_col * sizeof(double));
-    memcpy(col_lower, lp.col_lower_.data(), *num_col * sizeof(double));
-    memcpy(col_upper, lp.col_upper_.data(), *num_col * sizeof(double));
+    if (col_cost)
+      memcpy(col_cost, lp.col_cost_.data(), *num_col * sizeof(double));
+    if (col_lower)
+      memcpy(col_lower, lp.col_lower_.data(), *num_col * sizeof(double));
+    if (col_upper)
+      memcpy(col_upper, lp.col_upper_.data(), *num_col * sizeof(double));
   }
   if (*num_row > 0) {
-    memcpy(row_lower, lp.row_lower_.data(), *num_row * sizeof(double));
-    memcpy(row_upper, lp.row_upper_.data(), *num_row * sizeof(double));
+    if (row_lower)
+      memcpy(row_lower, lp.row_lower_.data(), *num_row * sizeof(double));
+    if (row_upper)
+      memcpy(row_upper, lp.row_upper_.data(), *num_row * sizeof(double));
   }
 
   // Nothing to do if one of the matrix dimensions is zero
@@ -1119,10 +1128,13 @@ HighsInt Highs_getHighsLpData(const HighsLp& lp, const HighsInt a_format,
          lp.a_matrix_.isRowwise())) {
       // Incumbent format is OK
       *num_nz = lp.a_matrix_.numNz();
-      memcpy(a_start, lp.a_matrix_.start_.data(),
-             num_start_entries * sizeof(HighsInt));
-      memcpy(a_index, lp.a_matrix_.index_.data(), *num_nz * sizeof(HighsInt));
-      memcpy(a_value, lp.a_matrix_.value_.data(), *num_nz * sizeof(double));
+      if (a_start)
+        memcpy(a_start, lp.a_matrix_.start_.data(),
+               num_start_entries * sizeof(HighsInt));
+      if (a_index)
+        memcpy(a_index, lp.a_matrix_.index_.data(), *num_nz * sizeof(HighsInt));
+      if (a_value)
+        memcpy(a_value, lp.a_matrix_.value_.data(), *num_nz * sizeof(double));
     } else {
       // Take a copy and transpose it
       HighsSparseMatrix local_matrix = lp.a_matrix_;
@@ -1134,13 +1146,16 @@ HighsInt Highs_getHighsLpData(const HighsLp& lp, const HighsInt a_format,
         local_matrix.ensureRowwise();
       }
       *num_nz = local_matrix.numNz();
-      memcpy(a_start, local_matrix.start_.data(),
-             num_start_entries * sizeof(HighsInt));
-      memcpy(a_index, local_matrix.index_.data(), *num_nz * sizeof(HighsInt));
-      memcpy(a_value, local_matrix.value_.data(), *num_nz * sizeof(double));
+      if (a_start)
+        memcpy(a_start, local_matrix.start_.data(),
+               num_start_entries * sizeof(HighsInt));
+      if (a_index)
+        memcpy(a_index, local_matrix.index_.data(), *num_nz * sizeof(HighsInt));
+      if (a_value)
+        memcpy(a_value, local_matrix.value_.data(), *num_nz * sizeof(double));
     }
   }
-  if (HighsInt(lp.integrality_.size())) {
+  if (HighsInt(lp.integrality_.size()) && integrality) {
     for (int iCol = 0; iCol < *num_col; iCol++)
       integrality[iCol] = HighsInt(lp.integrality_[iCol]);
   }
@@ -1163,9 +1178,12 @@ HighsInt Highs_getModel(const void* highs, const HighsInt a_format,
   const HighsHessian& hessian = ((Highs*)highs)->getModel().hessian_;
   if (hessian.dim_ > 0) {
     *q_num_nz = hessian.start_[*num_col];
-    memcpy(q_start, hessian.start_.data(), *num_col * sizeof(HighsInt));
-    memcpy(q_index, hessian.index_.data(), *q_num_nz * sizeof(HighsInt));
-    memcpy(q_value, hessian.value_.data(), *q_num_nz * sizeof(double));
+    if (q_start)
+      memcpy(q_start, hessian.start_.data(), *num_col * sizeof(HighsInt));
+    if (q_index)
+      memcpy(q_index, hessian.index_.data(), *q_num_nz * sizeof(HighsInt));
+    if (q_value)
+      memcpy(q_value, hessian.value_.data(), *q_num_nz * sizeof(double));
   }
   return kHighsStatusOk;
 }
@@ -1325,10 +1343,61 @@ void Highs_resetGlobalScheduler(HighsInt blocking) {
   Highs::resetGlobalScheduler(blocking != 0);
 }
 
+const void* Highs_getCallbackDataOutItem(const HighsCallbackDataOut* data_out,
+                                         const char* item_name) {
+  // Accessor function for HighsCallbackDataOut
+  //
+  // Remember that pointers in HighsCallbackDataOut don't need to be referenced!
+  if (!strcmp(item_name, kHighsCallbackDataOutLogTypeName)) {
+    return (void*)(&data_out->log_type);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutRunningTimeName)) {
+    return (void*)(&data_out->running_time);
+  } else if (!strcmp(item_name,
+                     kHighsCallbackDataOutSimplexIterationCountName)) {
+    return (void*)(&data_out->simplex_iteration_count);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutIpmIterationCountName)) {
+    return (void*)(&data_out->ipm_iteration_count);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutPdlpIterationCountName)) {
+    return (void*)(&data_out->pdlp_iteration_count);
+  } else if (!strcmp(item_name,
+                     kHighsCallbackDataOutObjectiveFunctionValueName)) {
+    return (void*)(&data_out->objective_function_value);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutMipNodeCountName)) {
+    return (void*)(&data_out->mip_node_count);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutMipPrimalBoundName)) {
+    return (void*)(&data_out->mip_primal_bound);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutMipDualBoundName)) {
+    return (void*)(&data_out->mip_dual_bound);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutMipGapName)) {
+    return (void*)(&data_out->mip_gap);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutMipSolutionName)) {
+    return (void*)(data_out->mip_solution);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolNumColName)) {
+    return (void*)(&data_out->cutpool_num_col);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolNumCutName)) {
+    return (void*)(&data_out->cutpool_num_cut);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolNumNzName)) {
+    return (void*)(&data_out->cutpool_num_nz);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolStartName)) {
+    return (void*)(data_out->cutpool_start);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolIndexName)) {
+    return (void*)(data_out->cutpool_index);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolValueName)) {
+    return (void*)(data_out->cutpool_value);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolLowerName)) {
+    return (void*)(data_out->cutpool_lower);
+  } else if (!strcmp(item_name, kHighsCallbackDataOutCutpoolUpperName)) {
+    return (void*)(data_out->cutpool_upper);
+  }
+  return nullptr;
+}
+
 // *********************
 // * Deprecated methods*
 // *********************
 
+const char* Highs_compilationDate(void) { return "Deprecated"; }
+
 HighsInt Highs_call(const HighsInt num_col, const HighsInt num_row,
                     const HighsInt num_nz, const double* col_cost,
                     const double* col_lower, const double* col_upper,
