@@ -393,6 +393,105 @@ void HighsLp::addRowNames(const std::string name, const HighsInt num_new_row) {
   }
 }
 
+void HighsLp::deleteColsFromVectors(
+    HighsInt& new_num_col, const HighsIndexCollection& index_collection) {
+  assert(ok(index_collection));
+  HighsInt from_k;
+  HighsInt to_k;
+  limits(index_collection, from_k, to_k);
+  // Initialise new_num_col in case none is removed due to from_k > to_k
+  new_num_col = this->num_col_;
+  if (from_k > to_k) return;
+
+  HighsInt delete_from_col;
+  HighsInt delete_to_col;
+  HighsInt keep_from_col;
+  HighsInt keep_to_col = -1;
+  HighsInt current_set_entry = 0;
+
+  HighsInt col_dim = this->num_col_;
+  new_num_col = 0;
+  bool have_names = (this->col_names_.size() != 0);
+  bool have_integrality = (this->integrality_.size() != 0);
+  for (HighsInt k = from_k; k <= to_k; k++) {
+    updateOutInIndex(index_collection, delete_from_col, delete_to_col,
+                     keep_from_col, keep_to_col, current_set_entry);
+    // Account for the initial columns being kept
+    if (k == from_k) new_num_col = delete_from_col;
+    if (delete_to_col >= col_dim - 1) break;
+    assert(delete_to_col < col_dim);
+    for (HighsInt col = keep_from_col; col <= keep_to_col; col++) {
+      this->col_cost_[new_num_col] = this->col_cost_[col];
+      this->col_lower_[new_num_col] = this->col_lower_[col];
+      this->col_upper_[new_num_col] = this->col_upper_[col];
+      if (have_names) this->col_names_[new_num_col] = this->col_names_[col];
+      if (have_integrality)
+        this->integrality_[new_num_col] = this->integrality_[col];
+      new_num_col++;
+    }
+    if (keep_to_col >= col_dim - 1) break;
+  }
+  this->col_cost_.resize(new_num_col);
+  this->col_lower_.resize(new_num_col);
+  this->col_upper_.resize(new_num_col);
+  if (have_names) this->col_names_.resize(new_num_col);
+}
+
+void HighsLp::deleteRowsFromVectors(
+    HighsInt& new_num_row, const HighsIndexCollection& index_collection) {
+  assert(ok(index_collection));
+  HighsInt from_k;
+  HighsInt to_k;
+  limits(index_collection, from_k, to_k);
+  // Initialise new_num_row in case none is removed due to from_k > to_k
+  new_num_row = this->num_row_;
+  if (from_k > to_k) return;
+
+  HighsInt delete_from_row;
+  HighsInt delete_to_row;
+  HighsInt keep_from_row;
+  HighsInt keep_to_row = -1;
+  HighsInt current_set_entry = 0;
+
+  HighsInt row_dim = this->num_row_;
+  new_num_row = 0;
+  bool have_names = (HighsInt)this->row_names_.size() > 0;
+  for (HighsInt k = from_k; k <= to_k; k++) {
+    updateOutInIndex(index_collection, delete_from_row, delete_to_row,
+                     keep_from_row, keep_to_row, current_set_entry);
+    if (k == from_k) {
+      // Account for the initial rows being kept
+      new_num_row = delete_from_row;
+    }
+    if (delete_to_row >= row_dim - 1) break;
+    assert(delete_to_row < row_dim);
+    for (HighsInt row = keep_from_row; row <= keep_to_row; row++) {
+      this->row_lower_[new_num_row] = this->row_lower_[row];
+      this->row_upper_[new_num_row] = this->row_upper_[row];
+      if (have_names) this->row_names_[new_num_row] = this->row_names_[row];
+      new_num_row++;
+    }
+    if (keep_to_row >= row_dim - 1) break;
+  }
+  this->row_lower_.resize(new_num_row);
+  this->row_upper_.resize(new_num_row);
+  if (have_names) this->row_names_.resize(new_num_row);
+}
+
+void HighsLp::deleteCols(const HighsIndexCollection& index_collection) {
+  HighsInt new_num_col;
+  this->deleteColsFromVectors(new_num_col, index_collection);
+  this->a_matrix_.deleteCols(index_collection);
+  this->num_col_ = new_num_col;
+}
+
+void HighsLp::deleteRows(const HighsIndexCollection& index_collection) {
+  HighsInt new_num_row;
+  this->deleteRowsFromVectors(new_num_row, index_collection);
+  this->a_matrix_.deleteRows(index_collection);
+  this->num_row_ = new_num_row;
+}
+
 void HighsLp::unapplyMods() {
   // Restore any non-semi types
   const HighsInt num_non_semi = this->mods_.save_non_semi_variable_index.size();
@@ -495,14 +594,13 @@ void HighsNameHash::form(const std::vector<std::string>& name) {
   size_t num_name = name.size();
   this->clear();
   for (size_t index = 0; index < num_name; index++) {
-    const bool duplicate = !this->name2index.emplace(name[index], index).second;
+    auto emplace_result = this->name2index.emplace(name[index], index);
+    const bool duplicate = !emplace_result.second;
     if (duplicate) {
       // Find the original and mark it as duplicate
-      auto search = this->name2index.find(name[index]);
-      assert(search != this->name2index.end());
+      auto& search = emplace_result.first;
       assert(int(search->second) < int(this->name2index.size()));
-      this->name2index.erase(search);
-      this->name2index.insert({name[index], kHashIsDuplicate});
+      search->second = kHashIsDuplicate;
     }
   }
 }
@@ -519,4 +617,17 @@ bool HighsNameHash::hasDuplicate(const std::vector<std::string>& name) {
   return has_duplicate;
 }
 
+void HighsNameHash::update(int index, const std::string& old_name,
+                           const std::string& new_name) {
+  this->name2index.erase(old_name);
+  auto emplace_result = this->name2index.emplace(new_name, index);
+  const bool duplicate = !emplace_result.second;
+  if (duplicate) {
+    // Find the original and mark it as duplicate
+    auto& search = emplace_result.first;
+    assert(int(search->second) < int(this->name2index.size()));
+    search->second = kHashIsDuplicate;
+  }
+}
+
 void HighsNameHash::clear() { this->name2index.clear(); }
