@@ -1231,6 +1231,9 @@ std::string utilModelStatusToString(const HighsModelStatus model_status) {
     case HighsModelStatus::kModelEmpty:
       return "Empty";
       break;
+    case HighsModelStatus::kMemoryLimit:
+      return "Memory limit reached";
+      break;
     case HighsModelStatus::kOptimal:
       return "Optimal";
       break;
@@ -1319,6 +1322,8 @@ HighsStatus highsStatusFromHighsModelStatus(HighsModelStatus model_status) {
       return HighsStatus::kError;
     case HighsModelStatus::kPostsolveError:
       return HighsStatus::kError;
+    case HighsModelStatus::kMemoryLimit:
+      return HighsStatus::kError;
     case HighsModelStatus::kModelEmpty:
       return HighsStatus::kOk;
     case HighsModelStatus::kOptimal:
